@@ -28,6 +28,9 @@ class Middlewares:
         "pack": [],
         "publish": [],
         "run": [],
+        "dev": [],
+        "invoke": [],
+        "eval": [],
     }
     _plugins_loaded = False
 
