@@ -8,7 +8,7 @@ from utils.project_details import ProjectDetails
 from utils.uipath_json import UiPathJson
 
 import uipath._cli.cli_pack as cli_pack
-from uipath._cli.cli_pack import pack
+from uipath._cli import cli
 
 
 class TestPack:
@@ -29,7 +29,7 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 0
             assert os.path.exists(
                 f".uipath/{project_details.name}.{project_details.version}.nupkg"
@@ -50,7 +50,7 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert (
                 "pyproject.toml is missing the required field: project.description."
@@ -72,13 +72,34 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert (
                 """Project authors cannot be empty. Please specify authors in pyproject.toml:\n    authors = [{ name = "John Doe" }]"""
                 in result.output
             )
 
+    def test_pyproject_missing_requires_python(
+        self,
+        runner: CliRunner,
+        temp_dir: str,
+        project_details: ProjectDetails,
+        uipath_json: UiPathJson,
+    ) -> None:
+        project_details.requires_python = None
+        with runner.isolated_filesystem(temp_dir=temp_dir):
+            with open("uipath.json", "w") as f:
+                f.write(uipath_json.to_json())
+            with open("pyproject.toml", "w") as f:
+                f.write(project_details.to_toml())
+
+            result = runner.invoke(cli, ["pack", "./"])
+            assert result.exit_code == 1
+            assert (
+                "'requires-python' field cannot be empty. Please specify it in pyproject.toml"
+                in result.output
+            )
+
     def test_pyproject_missing_project_name(
         self,
         runner: CliRunner,
@@ -93,7 +114,7 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert (
                 "Project name cannot be empty. Please specify a name in pyproject.toml."
@@ -114,7 +135,7 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert """Project name contains invalid character: '<'""" in result.output
 
@@ -132,7 +153,7 @@ class TestPack:
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert (
                 """Project description contains invalid character: '&'"""
@@ -146,7 +167,7 @@ class TestPack:
         with runner.isolated_filesystem(temp_dir=temp_dir):
             with open("pyproject.toml", "w") as f:
                 f.write(project_details.to_toml())
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert (
                 "uipath.json not found. Please run `uipath init` in the project directory."
@@ -161,7 +182,7 @@ class TestPack:
             with open("uipath.json", "w") as f:
                 f.write(uipath_json.to_json())
 
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
             assert result.exit_code == 1
             assert "pyproject.toml not found" in result.output
 
@@ -265,7 +286,7 @@ class TestPack:
                 f.write(binary_content)
             with open(binary_file_not_included, "w") as f:
                 f.write("---")
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
 
             assert result.exit_code == 0
             with zipfile.ZipFile(
@@ -303,7 +324,7 @@ class TestPack:
                 f.write("<root><child>text</child></root>")
             with open(random_file, "w") as f:
                 f.write("<root><child>text</child></root>")
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
 
             assert result.exit_code == 0
             with zipfile.ZipFile(
@@ -312,6 +333,30 @@ class TestPack:
                 assert f"content/{file_to_add}" in z.namelist()
                 assert f"content/{random_file}" not in z.namelist()
 
+    def test_include_subdir_files(
+        self,
+        runner: CliRunner,
+        temp_dir: str,
+        project_details: ProjectDetails,
+        uipath_json: UiPathJson,
+    ) -> None:
+        """Test generating operate.json and its content."""
+        with runner.isolated_filesystem(temp_dir=temp_dir):
+            with open("uipath.json", "w") as f:
+                f.write(uipath_json.to_json())
+            with open("pyproject.toml", "w") as f:
+                f.write(project_details.to_toml())
+            os.mkdir("subdir")
+            with open("subdir/should_be_included.py", "w") as f:
+                f.write('print("This file should be included in the .nupkg")')
+            result = runner.invoke(cli, ["pack", "./"])
+
+            assert result.exit_code == 0
+            with zipfile.ZipFile(
+                f".uipath/{project_details.name}.{project_details.version}.nupkg", "r"
+            ) as z:
+                assert "content/subdir/should_be_included.py" in z.namelist()
+
     def test_successful_pack(
         self,
         runner: CliRunner,
@@ -328,7 +373,7 @@ class TestPack:
             for entry in uipath_json.entry_points:
                 with open(f"{entry.file_path}.py", "w") as f:
                     f.write("#agent content")
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
 
             assert result.exit_code == 0
             with zipfile.ZipFile(
@@ -397,7 +442,7 @@ class TestPack:
                     f.write("# agent content")
 
             # Run pack command
-            result = runner.invoke(pack, ["./"])
+            result = runner.invoke(cli, ["pack", "./"])
 
             # Assert pack was successful
             assert result.exit_code == 0, f"Pack failed with output: {result.output}"
@@ -493,3 +538,79 @@ class TestPack:
                 assert "projectId" in operate_data
                 assert operate_data["targetRuntime"] == "python"
                 assert operate_data["targetFramework"] == "Portable"
+
+    def test_nupkg_contains_all_necessary_files(
+        self,
+        runner: CliRunner,
+        temp_dir: str,
+        project_details: ProjectDetails,
+        uipath_json: UiPathJson,
+    ) -> None:
+        with runner.isolated_filesystem(temp_dir=temp_dir):
+            with open("uipath.json", "w") as f:
+                f.write(uipath_json.to_json())
+            with open("pyproject.toml", "w") as f:
+                f.write(project_details.to_toml())
+            with open("uv.lock", "w") as f:
+                f.write("# uv.lock content")
+            for entry in uipath_json.entry_points:
+                with open(f"{entry.file_path}.py", "w") as f:
+                    f.write("# agent content")
+
+            result = runner.invoke(cli, ["pack", "./"])
+            assert result.exit_code == 0
+
+            nupkg_path = (
+                f".uipath/{project_details.name}.{project_details.version}.nupkg"
+            )
+            assert os.path.exists(nupkg_path)
+
+            # List of expected files in the package
+            expected_files = [
+                "content/uipath.json",
+                "content/pyproject.toml",
+                "content/operate.json",
+                "content/entry-points.json",
+                "content/bindings_v2.json",
+                "content/uv.lock",
+            ]
+
+            for entry in uipath_json.entry_points:
+                expected_files.append(f"content/{entry.file_path}.py")
+
+            with zipfile.ZipFile(nupkg_path, "r") as z:
+                actual_files = set(z.namelist())
+                for expected in expected_files:
+                    assert expected in actual_files, f"Missing {expected} in nupkg"
+
+    def test_no_uv_lock_with_nolock(
+        self,
+        runner: CliRunner,
+        temp_dir: str,
+        project_details: ProjectDetails,
+        uipath_json: UiPathJson,
+    ) -> None:
+        with runner.isolated_filesystem(temp_dir=temp_dir):
+            with open("uipath.json", "w") as f:
+                f.write(uipath_json.to_json())
+            with open("pyproject.toml", "w") as f:
+                f.write(project_details.to_toml())
+            with open("uv.lock", "w") as f:
+                f.write("# uv.lock content")
+            for entry in uipath_json.entry_points:
+                with open(f"{entry.file_path}.py", "w") as f:
+                    f.write("# agent content")
+
+            result = runner.invoke(cli, ["pack", "./", "--nolock"])
+            assert result.exit_code == 0
+
+            nupkg_path = (
+                f".uipath/{project_details.name}.{project_details.version}.nupkg"
+            )
+            assert os.path.exists(nupkg_path)
+
+            with zipfile.ZipFile(nupkg_path, "r") as z:
+                actual_files = set(z.namelist())
+                assert "content/uv.lock" not in actual_files, (
+                    "uv.lock should not be in nupkg when --nolock is used"
+                )
